/*
 * Core Interface for Kaspersky AntiVirus
 *
 * Copyright (C) Ries van Twisk, vscan@rvt.dds.nl, 2002
 * Copyright (C) Rainer Link, rainer@openantivirus.org, 2002
 *
 * This software is licensed under the GNU General Public License (GPL)
 * See: http://www.gnu.org/copyleft/gpl.html
 *
*/

#include "vscan-kavp.h"
#include "vscan-kavp_core.h"
#include "kavclient.h"
#include <includes.h>

extern BOOL verbose_file_logging;
extern fstring avpctl;


/* Scan a file */
/* return 
    0  everything is ok 
    -1 File contains a virii
    -2 socket is not open
    -3 request path file
    -4 response failes
*/


/* print a message via syslog */
void vscan_syslog(char *printMessage, ...)
{
        char printMsg[512];

        va_list argptr;
        va_start(argptr, printMessage);
        vsnprintf(printMsg, sizeof(printMsg)-1, printMessage, argptr);
        va_end(argptr);

        syslog(SYSLOG_PRIORITY, "%s", printMsg);
}


/* print a message via syslog */
void vscan_syslog_alert(char *printMessage, ...)
{
        char printMsg[512];

        va_list argptr;
        va_start(argptr, printMessage);
        vsnprintf(printMsg, sizeof(printMsg)-1, printMessage, argptr);
        va_end(argptr);

        syslog(SYSLOG_PRIORITY_ALERT, "%s", printMsg);
}


/*
  If virus is found, logs the filename/virusname into syslog
*/
void vscan_kavp_log_virus(char *infected_file, char *client_ip)
{
	// there seems no way to get the virus name :(        
	static char virusName[] = "UNKNOWN";

	vscan_syslog_alert("ALERT - Scan result: '%s' infected with virus '%s', client: '%s'", infected_file, virusName, client_ip);
        
}



int vscan_kavp_scanfile(char *scan_file, char* client_ip)
{
	int result;
	char* response;
	int exit_code;

	/* Check the socket */
	if (kavp_socket == 0) {
	    vscan_syslog("ERROR: connection to kavpdaemon was not open!\n");
    	    return -2;
	};	

	
	/* Send scan request to kavdaeon */
	if ( verbose_file_logging )
		vscan_syslog("ERROR: KAVRequestPath() scanning file [%s]\n", scan_file);

	if ( (result=KAVRequestPath(kavp_socket, scan_file, SILENT )) < 0) {
	    vscan_syslog("ERROR: KAVRequestMulti() failed (return code: [%d])\n", result);
	    return -3;
	};

	/* Receive status back about this file */	
	if ( (response=KAVResponse(kavp_socket, &exit_code, SILENT, 0)) ==0 ) {
	    vscan_syslog("ERROR: KAVResponse() failed (return code: [0])\n");
	    return -4;
	};	
    
	if ((exit_code & 0xff) - 0x30) {
	    vscan_kavp_log_virus(scan_file, client_ip);
	    return -1;	// Found a virus deny
	} else {
		if ( verbose_file_logging )
		    vscan_syslog("file %s is clean", scan_file);

	    return 0;	// Everything seems to be ok
	};
}

/* Initialize VSAPI */
void vscan_kavp_init(void)
{
    kavp_socket = KAVConnect(avpctl, SILENT);
    if ( kavp_socket < 0 ) {
	vscan_syslog("ERROR: KAVConnect() to socket %s failed (return code: [%d])\n", avpctl, kavp_socket);
    }
}

/* Bye, bye */
void vscan_kavp_end(void)
{
    int result;
    if (kavp_socket>0) {
	if ((result=KAVClose(kavp_socket, SILENT)) ==0 )
    	    vscan_syslog("Disconnected from kavdaemon.\n");	
	else
    	   vscan_syslog("ERROR :KAVClose() failed (return code: [%d])\n", result);	
	kavp_socket = -1;
    } else {
	vscan_syslog("Not closing a closed connection\n");
    };
}


